package com.sun.xml.internal.ws.client;

import java.util.Collections;
import java.util.Iterator;
import com.sun.xml.internal.ws.api.message.AttachmentSet;
import javax.activation.DataHandler;
import com.sun.xml.internal.ws.api.message.Attachment;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import com.sun.xml.internal.ws.api.message.Packet;
import java.util.AbstractMap;

public class ResponseContext extends AbstractMap<String, Object>
{
    private final Packet packet;
    private Set<Map.Entry<String, Object>> entrySet;
    
    public ResponseContext(Packet packet) {
        this.packet = packet;
    }
    
    @Override
    public boolean containsKey(Object key) {
        if (this.packet.supports(key)) {
            return this.packet.containsKey(key);
        }
        return this.packet.invocationProperties.containsKey(key) && !this.packet.getHandlerScopePropertyNames(true).contains(key);
    }
    
    @Override
    public Object get(Object key) {
        if (this.packet.supports(key)) {
            return this.packet.get(key);
        }
        if (this.packet.getHandlerScopePropertyNames(true).contains(key)) {
            return null;
        }
        Object value = this.packet.invocationProperties.get(key);
        if (key.equals("javax.xml.ws.binding.attachments.inbound")) {
            Map<String, DataHandler> atts = (Map)value;
            if (atts == null) {
                atts = new HashMap();
            }
            AttachmentSet attSet = this.packet.getMessage().getAttachments();
            for (Attachment att : attSet) {
                atts.put(att.getContentId(), att.asDataHandler());
            }
            return atts;
        }
        else {
            return value;
        }
    }
    
    @Override
    public Object put(String key, Object value) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public void putAll(Map<? extends String, ?> t) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Set<Map.Entry<String, Object>> entrySet() {
        if (this.entrySet == null) {
            Map<String, Object> r = new HashMap();
            r.putAll(this.packet.invocationProperties);
            r.keySet().removeAll(this.packet.getHandlerScopePropertyNames(true));
            r.putAll(this.packet.createMapView());
            this.entrySet = Collections.unmodifiableSet(r.entrySet());
        }
        return this.entrySet;
    }
}
